# -*- coding: utf-8 -*-

from odoo import api, fields, models
import pyodbc

class ResConfigSettings(models.TransientModel):
    _inherit = 'res.config.settings'

    it_import_edbs_server = fields.Char("Server", config_parameter='iteas_import.edbs_server')
    it_import_edbs_database = fields.Char("Database", config_parameter='iteas_import.edbs_database')
    it_import_edbs_username = fields.Char("User", config_parameter='iteas_import.edbs_username')
    it_import_edbs_password = fields.Char("Password", config_parameter='iteas_import.edbs_password')

    def import_edbs(self):
        cnxn = pyodbc.connect('DRIVER={ODBC Driver 17 for SQL Server};SERVER=' + self.it_import_edbs_server + ';DATABASE=' + self.it_import_edbs_database + ';UID=' + self.it_import_edbs_username + ';PWD=' + self.it_import_edbs_password)
        cursor = cnxn.cursor()

        company_cids = []
        contact_cids = []
        supplier_cids = []
        supplier_contact_cids = []
        cids2oids = {}
        sids2oids = {}

        companies = self.env['res.partner'].search([['is_company', '=', True]])
        for comp in companies:
            if comp.it_customer_id:
                company_cids.append(comp.it_customer_id)

        contacts = self.env['res.partner'].search([['is_company', '=', False]])
        for contact in contacts:
            if contact.it_customer_id:
                contact_cids.append(contact.it_customer_id)

        suppliers = self.env['res.partner'].search([['is_company', '=', True], ['supplier_rank', '=', 1]])
        for supplier in suppliers:
            if supplier.it_customer_id:
                supplier_cids.append(supplier.it_customer_id)

        supplier_contacts = self.env['res.partner'].search([['is_company', '=', False], ['supplier_rank', '=', 1]])
        for supplier_contact in supplier_contacts:
            if supplier_contact.it_customer_id:
                supplier_contact_cids.append(supplier_contact.it_customer_id)


        # Get countryname to countryid mapping
        co2coid = {}
        countries = self.env['res.country'].with_context(lang='de_DE').search([])
        for country in countries:
            co2coid[country.name] = country.id

        # Get title to titleid mapping
        ti2tiid = {}
        titles = self.env['res.partner.title'].with_context(lang='de_DE').search([])
        for title in titles:
            ti2tiid[title.name] = title.id

        if "Herr" in ti2tiid:
            ti2tiid["Herrn"] = ti2tiid["Herr"]

        # Get CustomerNr to VAT/Infotext mapping
        conr2vat = {}
        conr2info = {}
        cursor.execute("SELECT S_VATNO, LS_TEXT, S_CUSTNO from [EDBS_Iteas].[sao].[customer_m] WHERE d_Optional11 IS NULL")
        for row in cursor:
            conr2vat[row.S_CUSTNO] = row.S_VATNO
            conr2info[row.S_CUSTNO] = row.LS_TEXT

        # Write customer/supplier company records
        cursor.execute("SELECT * from [EDBS_Iteas].[sao].[aaEDVOTRS] WHERE ((BOID1='customer_customer' AND BOID2='customer_customer') or (BOID1='supplier_supplier' AND BOID2='supplier_supplier')) and INAKTIVDATUM IS NULL")
        for row in cursor:
            if not row.ROWID1:
                print("Skipping row, now ROWID1 found, data:")
                print(row)
                continue

            name = row.COMPANY if row.COMPANY else row.FIRSTNAME + " " + row.NAME
            name2 = row.FIRSTNAME if row.COMPANY else ""

            data = {
                'it_customer_id': row.ROWID1,
                'name': name,
                'it_name2': name2,
                'is_company': True,
                'street': row.STREET,
                'zip': row.ZIPCODENO,
                'city': row.TOWN,
                'website': row.WWW,
                'email': row.EMAIL,
                'phone': row.PHONE,
                'mobile': row.MOBILE,
                'it_customer_nr': row.PERSONNO,
                'function': row.FUNKTION,
                'comment': row.NICKNAME
            }

            if row.COUNTRY in co2coid:
                data['country_id'] = co2coid[row.COUNTRY]

            if row.PERSONNO in conr2vat:
                data['vat'] = conr2vat[row.PERSONNO]

            if row.PERSONNO in conr2info:
                data['it_comment2'] = conr2info[row.PERSONNO]

            # Customer
            if row.BOID1 == "customer_customer":
                data["customer_rank"] = 1

                # Record exists, updating
                if row.ROWID1 in company_cids:
                    print("update customer:", name)

                    obj = self.env['res.partner'].search([['it_customer_id', '=', row.ROWID1], ['is_company', '=', True], ['supplier_rank', '=', 0]])
                    obj.update(data)
                    print(obj, row.ROWID1)
                    cids2oids[row.ROWID1] = obj.id

                # Record does not exist, creating
                else:
                    print("insert new customer:", name)

                    obj = self.env['res.partner'].create(data)
                    cids2oids[row.ROWID1] = obj.id

            # Supplier
            elif row.BOID1 == "supplier_supplier":
                data["supplier_rank"] = 1

                # Record exists, updating
                if row.ROWID1 in supplier_cids:
                    print("update supplier:", name)

                    obj = self.env['res.partner'].search([['it_customer_id', '=', row.ROWID1], ['is_company', '=', True], ['supplier_rank', '=', 1]])
                    obj.update(data)
                    print(obj, row.ROWID1)
                    sids2oids[row.ROWID1] = obj.id

                # Record does not exist, creating
                else:
                    print("insert new supplier:", name)

                    obj = self.env['res.partner'].create(data)
                    sids2oids[row.ROWID1] = obj.id

        # Write customer/supplier contact records
        cursor.execute("SELECT * from [EDBS_Iteas].[sao].[aaEDVOTRS] WHERE ((BOID1='customer_customer' AND BOID2='contact_contact') or (BOID1='supplier_supplier' AND BOID2='contact_contact')) and INAKTIVDATUM IS NULL")
        for row in cursor:
            if not row.ROWID1:
                print("Skipping row, now ROWID1 found, data:")
                print(row)
                continue

            if row.FIRSTNAME and row.NAME:
                name = row.FIRSTNAME + " " + row.NAME
            else:
                name = row.NAME

            function = row.FUNKTION if row.FUNKTION.strip() else row.ABTEILUNG

            data = {
                'it_customer_id': row.ROWID2,
                'name': name,
                'is_company': False,
                'street': row.STREET,
                'zip': row.ZIPCODENO,
                'city': row.TOWN,
                'website': row.WWW,
                'email': row.EMAIL,
                'phone': row.PHONE,
                'mobile': row.MOBILE,
                'it_customer_nr': row.PERSONNO,
                'function': function,
                'comment': row.NICKNAME,
            }

            if row.COUNTRY in co2coid:
                data['country_id'] = co2coid[row.COUNTRY]

            if row.ABBREVIATION in ti2tiid:
                data['title'] = ti2tiid[row.ABBREVIATION]

            if row.PERSONNO in conr2info:
                data['it_comment2'] = conr2info[row.PERSONNO]

            # Customer
            if row.BOID1 == "customer_customer":
                data["customer_rank"] = 1

                # Link company if existing
                if row.ROWID1 in cids2oids:
                    data["parent_id"] = cids2oids[row.ROWID1]

                # Record exists, updating
                if row.ROWID2 in contact_cids:
                    print("update customer contact:", name)

                    obj = self.env['res.partner'].search([['it_customer_id', '=', row.ROWID2], ['is_company', '=', False], ['supplier_rank', '=', 0]])
                    obj.update(data)

                # Record does not exist, creating
                else:
                    print("insert new customer contact:", name)

                    self.env['res.partner'].create(data)

            # Supplier
            elif row.BOID1 == "supplier_supplier":
                data["supplier_rank"] = 1

                # Link company if existing
                if row.ROWID1 in sids2oids:
                    data["parent_id"] = sids2oids[row.ROWID1]

                # Record exists, updating
                if row.ROWID2 in supplier_contact_cids:
                    print("update supplier contact:", name)

                    obj = self.env['res.partner'].search([['it_customer_id', '=', row.ROWID2], ['is_company', '=', False], ['supplier_rank', '=', 1]])
                    obj.update(data)

                # Record does not exist, creating
                else:
                    print("insert new supplier contact:", name)

                    self.env['res.partner'].create(data)

        self.env.cr.commit()